@@ -198,6 +198,7 @@ module:hook("admin/repl-input", function (event)
 	if not ok then
 		event.origin.send(st.stanza("repl-result", { type = "error" }):text(err));
 	end
+	return true;
 end);
 
 -- Console commands --
@@ -828,7 +829,7 @@ available_columns = {
 		width = 11;
 		mapper = function(cert_status, session)
 			if cert_status then return capitalize(cert_status); end
-			if session.cert_chain_status == "Invalid" then
+			if session.cert_chain_status == "invalid" then
 				local cert_errors = set.new(session.cert_chain_errors[1]);
 				if cert_errors:contains("certificate has expired") then
 					return "Expired";
