@@ -1,7 +1,7 @@
 -- Prosody IM
 -- Copyright (C) 2008-2010 Matthew Wild
 -- Copyright (C) 2008-2010 Waqas Hussain
--- 
+--
 -- This project is MIT/X11 licensed. Please see the
 -- COPYING file in the source package for more information.
 --
@@ -10,16 +10,15 @@
 local st = require "util.stanza"
 
 local jid_split = require "util.jid".split;
+local jid_resource = require "util.jid".resource;
 local jid_prep = require "util.jid".prep;
-local t_concat = table.concat;
 local tonumber = tonumber;
-local pairs, ipairs = pairs, ipairs;
+local pairs = pairs;
 
 local rm_load_roster = require "core.rostermanager".load_roster;
 local rm_remove_from_roster = require "core.rostermanager".remove_from_roster;
 local rm_add_to_roster = require "core.rostermanager".add_to_roster;
 local rm_roster_push = require "core.rostermanager".roster_push;
-local core_post_stanza = prosody.core_post_stanza;
 
 module:add_feature("jabber:iq:roster");
 
@@ -36,15 +35,15 @@ module:hook("iq/self/jabber:iq:roster:query", function(event)
 
 	if stanza.attr.type == "get" then
 		local roster = st.reply(stanza);
-		
+
 		local client_ver = tonumber(stanza.tags[1].attr.ver);
 		local server_ver = tonumber(session.roster[false].version or 1);
-		
+
 		if not (client_ver and server_ver) or client_ver ~= server_ver then
 			roster:query("jabber:iq:roster");
 			-- Client does not support versioning, or has stale roster
 			for jid, item in pairs(session.roster) do
-				if jid ~= "pending" and jid then
+				if jid then
 					roster:tag("item", {
 						jid = jid,
 						subscription = item.subscription,
@@ -52,7 +51,7 @@ module:hook("iq/self/jabber:iq:roster:query", function(event)
 						name = item.name,
 					});
 					for group in pairs(item.groups) do
-						roster:tag("group"):text(group):up();
+						roster:text_tag("group", group);
 					end
 					roster:up(); -- move out from item
 				end
@@ -64,26 +63,19 @@ module:hook("iq/self/jabber:iq:roster:query", function(event)
 	else -- stanza.attr.type == "set"
 		local query = stanza.tags[1];
 		if #query.tags == 1 and query.tags[1].name == "item"
-				and query.tags[1].attr.xmlns == "jabber:iq:roster" and query.tags[1].attr.jid
-				-- Protection against overwriting roster.pending, until we move it
-				and query.tags[1].attr.jid ~= "pending" then
+				and query.tags[1].attr.xmlns == "jabber:iq:roster" and query.tags[1].attr.jid then
 			local item = query.tags[1];
 			local from_node, from_host = jid_split(stanza.attr.from);
 			local jid = jid_prep(item.attr.jid);
-			local node, host, resource = jid_split(jid);
-			if not resource and host then
+			if jid and not jid_resource(jid) then
 				if jid ~= from_node.."@"..from_host then
 					if item.attr.subscription == "remove" then
 						local roster = session.roster;
 						local r_item = roster[jid];
 						if r_item then
-							local to_bare = node and (node.."@"..host) or host; -- bare JID
-							if r_item.subscription == "both" or r_item.subscription == "from" or (roster.pending and roster.pending[jid]) then
-								core_post_stanza(session, st.presence({type="unsubscribed", from=session.full_jid, to=to_bare}));
-							end
-							if r_item.subscription == "both" or r_item.subscription == "to" or r_item.ask then
-								core_post_stanza(session, st.presence({type="unsubscribe", from=session.full_jid, to=to_bare}));
-							end
+							module:fire_event("roster-item-removed", {
+								username = from_node, jid = jid, item = r_item, origin = session, roster = roster,
+							});
 							local success, err_type, err_cond, err_msg = rm_remove_from_roster(session, jid);
 							if success then
 								session.send(st.reply(stanza));
@@ -103,12 +95,10 @@ module:hook("iq/self/jabber:iq:roster:query", function(event)
 						else
 							r_item.subscription = "none";
 						end
-						for _, child in ipairs(item) do
-							if child.name == "group" then
-								local text = t_concat(child);
-								if text and text ~= "" then
-									r_item.groups[text] = true;
-								end
+						for group in item:childtags("group") do
+							local text = group:get_text();
+							if text then
+								r_item.groups[text] = true;
 							end
 						end
 						local success, err_type, err_cond, err_msg = rm_add_to_roster(session, jid, r_item);
@@ -140,16 +130,19 @@ end);
 
 module:hook_global("user-deleted", function(event)
 	local username, host = event.username, event.host;
+	local origin = event.origin or prosody.hosts[host];
 	if host ~= module.host then return end
-	local bare = username .. "@" .. host;
 	local roster = rm_load_roster(username, host);
 	for jid, item in pairs(roster) do
-		if jid and jid ~= "pending" then
-			if item.subscription == "both" or item.subscription == "from" or (roster.pending and roster.pending[jid]) then
-				module:send(st.presence({type="unsubscribed", from=bare, to=jid}));
-			end
-			if item.subscription == "both" or item.subscription == "to" or item.ask then
-				module:send(st.presence({type="unsubscribe", from=bare, to=jid}));
+		if jid then
+			module:fire_event("roster-item-removed", {
+				username = username, jid = jid, item = item, roster = roster, origin = origin,
+			});
+		else
+			for pending_jid in pairs(item.pending) do
+				module:fire_event("roster-item-removed", {
+					username = username, jid = pending_jid, roster = roster, origin = origin,
+				});
 			end
 		end
 	end
