@@ -130,10 +130,13 @@ function httpstream.new(success_cb, error_cb, parser_type, options_cb)
 							partial = true;
 						};
 					end
-					if len and len > bodylimit then
+					if not len or len > bodylimit then
 						-- Early notification, for redirection
 						success_cb(packet);
-						if not packet.body_sink then error = true; return error_cb("content-length-limit-exceeded"); end
+						if not packet.body_sink and (len and len > bodylimit) then
+							error = true;
+							return error_cb("content-length-limit-exceeded");
+						end
 					end
 					if chunked and not packet.body_sink then
 						success_cb(packet);
@@ -150,6 +153,7 @@ function httpstream.new(success_cb, error_cb, parser_type, options_cb)
 						if not chunk_size then return; end
 						chunk_size = chunk_size and tonumber(chunk_size, 16);
 						if not chunk_size then error = true; return error_cb("invalid-chunk-size"); end
+
 						if chunk_size == 0 and chunk_header:find("\r\n\r\n", chunk_start-2, true) then
 							local body_buffer = packet.body_buffer;
 							if body_buffer then
@@ -165,8 +169,8 @@ function httpstream.new(success_cb, error_cb, parser_type, options_cb)
 							state, chunked = nil, nil;
 							packet.partial = nil;
 							success_cb(packet);
-						elseif buffer:length() - chunk_start - 2 >= chunk_size then -- we have a chunk
-							buffer:discard(chunk_start - 1); -- TODO verify that it's not off-by-one
+						elseif buffer:length() - chunk_start - 1 >= chunk_size then -- we have a chunk
+							buffer:discard(chunk_start - 1);
 							(packet.body_sink or packet.body_buffer):write(buffer:read(chunk_size));
 							buffer:discard(2); -- CRLF
 						else -- Partial chunk remaining
@@ -174,9 +178,11 @@ function httpstream.new(success_cb, error_cb, parser_type, options_cb)
 						end
 					elseif packet.body_sink then
 						local chunk = buffer:read_chunk(len);
-						while chunk and len > 0 do
+						while chunk and (not len or len > 0) do
 							if packet.body_sink:write(chunk) then
-								len = len - #chunk;
+								if len then
+									len = len - #chunk;
+								end
 								chunk = buffer:read_chunk(len);
 							else
 								error = true;
@@ -188,9 +194,9 @@ function httpstream.new(success_cb, error_cb, parser_type, options_cb)
 							packet.partial = nil;
 							success_cb(packet);
 						end
-					elseif buffer:length() >= len then
+					elseif not len or buffer:length() >= len then -- or not len
 						assert(not chunked)
-						packet.body = buffer:read(len) or "";
+						packet.body = len and buffer:read(len) or buffer:read_chunk() or "";
 						state = nil;
 						packet.partial = nil;
 						success_cb(packet);
