@@ -34,8 +34,12 @@ local _ENV = nil;
 local stanza_mt = { __name = "stanza" };
 stanza_mt.__index = stanza_mt;
 
+-- Basic check for valid XML character data.
+-- Disallow control characters.
+-- Tab U+09 and newline U+0A are allowed.
+-- For attributes, allow the \1 separator between namespace and name.
 local function valid_xml_cdata(str, attr)
-	return not s_find(str, attr and "[^\1\9\10\13\20-~\128-\247]" or "[^\9\10\13\20-~\128-\247]");
+	return not s_find(str, attr and "[^\1\9\10\13\20-\255]" or "[^\9\10\13\20-\255]");
 end
 
 local function check_name(name, name_type)
@@ -164,6 +168,7 @@ function stanza_mt:get_child(name, xmlns)
 			return child;
 		end
 	end
+	return nil;
 end
 
 function stanza_mt:get_child_text(name, xmlns)
@@ -178,12 +183,14 @@ function stanza_mt:child_with_name(name)
 	for _, child in ipairs(self.tags) do
 		if child.name == name then return child; end
 	end
+	return nil;
 end
 
 function stanza_mt:child_with_ns(ns)
 	for _, child in ipairs(self.tags) do
 		if child.attr.xmlns == ns then return child; end
 	end
+	return nil;
 end
 
 function stanza_mt:get_child_with_attr(name, xmlns, attr_name, attr_value, normalize)
@@ -192,6 +199,7 @@ function stanza_mt:get_child_with_attr(name, xmlns, attr_name, attr_value, norma
 			return tag;
 		end
 	end
+	return nil;
 end
 
 function stanza_mt:children()
@@ -350,6 +358,7 @@ function stanza_mt.get_text(t)
 	if #t.tags == 0 then
 		return t_concat(t);
 	end
+	return nil;
 end
 
 function stanza_mt.get_error(stanza)
